/*******************************************************************************
 * Copyright (c) 2009 Information-technology Promotion Agency, Japan.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package benten.cat.glossary.ui.editor;

import java.util.List;

import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContentAssistProcessor;
import org.eclipse.jface.text.contentassist.IContextInformation;
import org.eclipse.jface.text.contentassist.IContextInformationValidator;
import org.eclipse.swt.graphics.Point;
import org.eclipse.wst.sse.ui.internal.contentassist.CustomCompletionProposal;

import benten.cat.glossary.core.BentenGlossaryEngine;
import benten.cat.glossary.core.BentenGlossarySearchResult;
import benten.cat.glossary.ui.CatGlossaryUiPlugin;

/**
 * 用語集コンテンツ・アシスト・プロセッサーです。
 * 
 * ★基本設計「翻訳支援機能: 翻訳中間形式エディター機能: 用語対訳による補完」に対応します。
 */
@SuppressWarnings("restriction")
public class GlossaryAssistProcessor implements IContentAssistProcessor {

	public GlossaryAssistProcessor() {
	}

	public ICompletionProposal[] computeCompletionProposals(ITextViewer viewer, int offset) {
		String source = viewer.getDocument().get();
		Point selectedRangePoint = viewer.getSelectedRange();
		String target = null;
		int replacementOffset;

		if (hasSelection(selectedRangePoint)) {
			target = source.substring(selectedRangePoint.x, selectedRangePoint.x + selectedRangePoint.y);
			replacementOffset = selectedRangePoint.x + selectedRangePoint.y;
		} else {
			target = getTargetString(source, offset);
			replacementOffset = offset;
		}

		BentenGlossaryEngine glossaryEngine = CatGlossaryUiPlugin.getDefault().getGlossaryEngine();
		List<BentenGlossarySearchResult> glossaryEntries = glossaryEngine.search(target);

		ICompletionProposal[] proposals = new ICompletionProposal[glossaryEntries.size()];

		for (int i = 0; i < glossaryEntries.size(); i++) {
			BentenGlossarySearchResult glossaryEntry = glossaryEntries.get(i);
			String srcText = glossaryEntry.getSource();
			String replacement = glossaryEntry.getTarget();
			String comment = glossaryEntry.getNote();
			proposals[i] = new CustomCompletionProposal(replacement, replacementOffset - target.length(), target
					.length(), replacement.length(), null, srcText + " - " + replacement, null, comment, 0, false); //$NON-NLS-1$
		}
		return proposals;
	}

	public IContextInformation[] computeContextInformation(ITextViewer viewer, int offset) {
		return null;
	}

	public char[] getCompletionProposalAutoActivationCharacters() {
		return null;
	}

	public char[] getContextInformationAutoActivationCharacters() {
		return null;
	}

	public IContextInformationValidator getContextInformationValidator() {
		return null;
	}

	public String getErrorMessage() {
		return null;
	}

	protected boolean hasSelection(Point selectedRangePoint) {
		return selectedRangePoint.y > 0;
	}

	protected String getTargetString(String source, int offset) {
		String beforeOffset = source.substring(0, offset);
		String targetString = beforeOffset.replaceFirst("(?s)(?i)([a-z-\\.]*[^a-z-\\.]*)*?([a-z-\\.]+[^a-z-\\.]*)$", //$NON-NLS-1$
				"$2"); //$NON-NLS-1$
		return targetString;
	}
}
